import React from 'react'
import raf from 'raf'
import TopPanel from './components/panel-top'
import MiddlePanel from './components/panel-middle'
import BottomPanel from './components/panel-bottom'
import Background from './components/background'
import environments from './components/environments'

export default class Nav extends React.Component {
  constructor (props) {
    super(props)

    this.state = {
      isMobile: false,
      isOpen: false,
      activeSection: -1,
      breakpoint: 895
    }

    this.toggle = this.toggle.bind(this)
    this.setActiveSection = this.setActiveSection.bind(this)
  }

  componentDidMount () {
    this.setState({ isMobile: window.innerWidth <= this.state.breakpoint })

    window.addEventListener('resize', () => {
      if (this.rafId) return

      this.rafId = raf(() => {
        const { isMobile, breakpoint } = this.state
        const { innerWidth } = window

        if (isMobile && innerWidth > breakpoint) {
          this.setState({ isMobile: false, isOpen: false, activeSection: -1 })
          document.body.classList.toggle('no-scroll', false)
        } else if (!isMobile && innerWidth <= breakpoint) {
          this.setState({ isMobile: true, activeSection: -1 })
        }

        this.rafId = undefined
      })
    })
  }

  setActiveSection (activeSection) {
    this.setState({ activeSection })
  }

  toggle (isOpen) {
    isOpen = isOpen || !this.state.isOpen

    if (this.state.isMobile) {
      document.body.classList.toggle('no-scroll', isOpen)
    }

    this.setState({ isOpen })
  }

  render () {
    const { isMobile, isOpen, activeSection } = this.state
    const { environment, topSections, sections, subsections } = this.props

    return (
      <nav className='nav-panel flex flex--center-y flex--wrap'>
        <TopPanel
          downloadJmpValue={ this.props.downloadJmpValue || environments[environment].downloadJmpValue }
          includeSearch={ this.props.includeSearch }
          host={ environment === 'com' ? '' : 'https://www.mongodb.com' }
          sections={ topSections || environments[environment].topSections }
          searchUrl={ this.props.searchUrl }
          contactUrl={ this.props.contactUrl } />
        <MiddlePanel
          { ...this.state }
          logoUrl={ this.props.logoUrl || environments[environment].logoUrl }
          tagline={ this.props.tagline || environments[environment].tagline }
          logoLink={ this.props.logoLink || environments[environment].logoLink }
          sections={ sections || environments[environment].sections }
          toggle={ this.toggle }
          setActiveSection={ this.setActiveSection } />
        <Background
          isMobile={ isMobile }
          setActiveSection={ this.setActiveSection }
          isOpaque={ isOpen || activeSection !== -1 || this.props.isOpaque }
          isOpen={ isOpen || activeSection !== -1 }>
          <BottomPanel
            { ...this.state }
            midSections={ sections || environments[environment].sections }
            sections={ subsections || environments[environment].subsections } />
        </Background>
      </nav>
    )
  }
}

const { bool, string, array } = React.PropTypes

Nav.propTypes = {
  isOpaque: bool,
  environment: string,
  includeSearch: bool,
  searchUrl: string,
  contactUrl: string,
  downloadJmpValue: string,
  logoUrl: string,
  logoLink: string,
  tagline: string,
  topSections: array,
  sections: array,
  subsections: array
}

Nav.defaultProps = {
  environment: 'com'
}
